@@ -105,6 +105,27 @@ static pthread_mutex_t searchLock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t compressorLock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t regexLock = PTHREAD_MUTEX_INITIALIZER;
 
+template<typename T>
+inline std::string _tostring(const T& value)
+{
+    std::ostringstream stream;
+    stream << value;
+    return stream.str();
+}
+
+
+std::pair<kiwix::Reader*, kiwix::Searcher*>
+get_from_humanReadableBookId(const std::string& humanReadableBookId) {
+  kiwix::Searcher* searcher
+        = searchers.find(humanReadableBookId) != searchers.end()
+              ? searchers.find(humanReadableBookId)->second
+              : globalSearcher;
+  kiwix::Reader* reader = readers.find(humanReadableBookId) != readers.end()
+                                ? readers.find(humanReadableBookId)->second
+                                : NULL;
+  return std::pair<kiwix::Reader*, kiwix::Searcher*>(reader, searcher);
+}
+
 /* Try to get the mimeType from the file extension */
 static std::string getMimeTypeForFile(const std::string& filename)
 {
@@ -138,7 +159,10 @@ static bool startswith(const std::string& base, const std::string& start)
 
 void introduceTaskbar(string& content, const string& humanReadableBookId)
 {
+  string zimTitle;
+
   pthread_mutex_lock(&regexLock);
+
   if (!noSearchBarFlag) {
     content = appendToFirstOccurence(
         content,
@@ -160,10 +184,18 @@ void introduceTaskbar(string& content, const string& humanReadableBookId)
              RESOURCE::taskbar_html_part,
              humanReadableBookId,
              "__CONTENT__"));
+
+      auto reader = get_from_humanReadableBookId(humanReadableBookId).first;
+      if (reader != nullptr) {
+        zimTitle = reader->getTitle();
+      }
     }
-    content = replaceRegex(content, rootLocation, "__ROOT_LOCATION__");
-    content = replaceRegex(content, replaceRegex(humanReadableBookId, "%26", "&"), "__CONTENT_ESCAPED__");
   }
+
+  content = replaceRegex(content, rootLocation, "__ROOT_LOCATION__");
+  content = replaceRegex(content, replaceRegex(zimTitle, "%26", "&"), "__ZIM_TITLE__");
+  content = replaceRegex(content, replaceRegex(humanReadableBookId, "%26", "&"), "__CONTENT_ESCAPED__");
+
   pthread_mutex_unlock(&regexLock);
 }
 
@@ -352,7 +384,7 @@ static struct MHD_Response* build_callback_response_from_entry(
   MHD_add_response_header(response, MHD_HTTP_HEADER_CONTENT_RANGE, oss.str().c_str());
 
   MHD_add_response_header(response, MHD_HTTP_HEADER_CONTENT_LENGTH,
-    std::to_string(range_len).c_str());
+    _tostring(range_len).c_str());
 
   /* Specify the mime type */
   MHD_add_response_header(
@@ -369,18 +401,6 @@ static struct MHD_Response* build_callback_response_from_entry(
   return response;
 }
 
-std::pair<kiwix::Reader*, kiwix::Searcher*>
-get_from_humanReadableBookId(const std::string& humanReadableBookId) {
-  kiwix::Searcher* searcher
-        = searchers.find(humanReadableBookId) != searchers.end()
-              ? searchers.find(humanReadableBookId)->second
-              : globalSearcher;
-  kiwix::Reader* reader = readers.find(humanReadableBookId) != readers.end()
-                                ? readers.find(humanReadableBookId)->second
-                                : NULL;
-  return std::pair<kiwix::Reader*, kiwix::Searcher*>(reader, searcher);
-}
-
 static struct MHD_Response* handle_meta(RequestContext* request)
 {
   std::string humanReadableBookId;
@@ -443,7 +463,7 @@ static struct MHD_Response* handle_suggest(RequestContext* request)
     humanReadableBookId = request->get_argument("content");
     term = request->get_argument("term");
   } catch (const std::out_of_range&) {
-    return build_homepage(request);
+    return build_404(request, "");
   }
 
   if (isVerbose.load()) {
@@ -628,12 +648,12 @@ static struct MHD_Response* handle_random(RequestContext* request)
   try {
     humanReadableBookId = request->get_argument("content");
   } catch (const std::out_of_range&) {
-    return build_homepage(request);
+    return build_404(request, "");
   }
 
   auto reader = get_from_humanReadableBookId(humanReadableBookId).first;
   if (reader == nullptr) {
-    return build_homepage(request);
+    return build_404(request, "");
   }
 
   try {
@@ -728,7 +748,11 @@ static struct MHD_Response* handle_content(RequestContext* request)
 
   auto reader = get_from_humanReadableBookId(humanReadableBookId).first;
   if (reader == nullptr) {
-    return build_homepage(request);
+    if (humanReadableBookId.size() == 0){
+        return build_homepage(request);
+    } else {
+        return build_404(request, "");
+    }
   }
 
   auto urlStr = request->get_url().substr(humanReadableBookId.size()+1);
@@ -842,7 +866,7 @@ static int accessHandlerCallback(void* cls,
   request.httpResponseCode = request.has_range() ? MHD_HTTP_PARTIAL_CONTENT : MHD_HTTP_OK;
 
   if (! request.is_valid_url()) {
-    response = build_homepage(&request);
+    response = build_404(&request, "");
   } else {
     if (startswith(request.get_url(), "/skin/")) {
       response = handle_skin(&request);
